package gui.components;

import java.awt.Graphics;
import java.awt.Point;
import java.util.List;
import java.util.LinkedList;

import gui.Drawable;
import logic.LogicalComponent;

public abstract class DrawableComponentAbstract implements DrawableComponent {

	LogicalComponent component;
	int x, y;
	String args[];
	boolean printInPortName;
	boolean printOutPortName;

	List<Drawable> drawables;

	public DrawableComponentAbstract(LogicalComponent component, int x, int y, String[] args, boolean printInPortName,
			boolean printOutPortName) {
		this.component = component;
		this.x = x;
		this.y = y;
		this.args = args;
		this.printInPortName = printInPortName;
		this.printOutPortName = printOutPortName;

		this.drawables = new LinkedList<Drawable>();
	}

	@Override
	public Point getCorner(int corner) {
		switch (corner) {
		case TOPLEFT:
			return new Point(x, y);
		case TOPRIGHT:
			return new Point(x + getWidth(), y);
		case BOTOMLEFT:
			return new Point(x, y + getHeight());
		case BOTOMRIGHT:
			return new Point(x + getWidth(), y + getHeight());
		}
		return null;
	}

	@Override
	public void draw(Graphics g) {
		for (Drawable drawable : drawables) {
			drawable.draw(g);
		}
	}

	@Override
	public void update() {
		for (Drawable drawable : drawables) {
			drawable.update();
		}
	}

	@Override
	public List<Drawable> getDrawables() {
		return drawables;
	}
}
