package gui.components;

import gui.Drawable;
import gui.GuiLine;
import gui.GuiPinLabel;
import gui.GuiPinLine;
import gui.GuiTextLabel;

import java.awt.Point;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import logic.LogicalComponent;
import logic.components.IntToBools;
import util.DrawUtils;

public class IntToBoolsGui extends DrawableComponentAbstract {

	Point inPortPosition;

	Point[] outPortPositions;

	public IntToBoolsGui(LogicalComponent i2b, int x, int y, String[] args,
			boolean printOutPortName) {
		this(i2b, x, y, args, true, printOutPortName);
	}

	public IntToBoolsGui(LogicalComponent i2b, int x, int y, String[] args,
			boolean printInPortName, boolean printOutPortName) {
		super(i2b, x, y, args, printInPortName, printOutPortName);

		inPortPosition = null;
		outPortPositions = new Point[i2b.getOutPins().length];
	}

	private static final int DISTANCE = 15;

	@Override
	public List<Drawable> getDrawables() {
		List<Drawable> gui = new LinkedList<Drawable>();

		IntToBools i2b = (IntToBools) component;

		GuiLine line; // Pomocna promenljiva
		List<List<Point>> sections;
		List<Point> points;

		int inNum = 1;
		int outNum = i2b.getOutPins().length;
		int heihgt = (outNum + 1) * DISTANCE;
		int width = 20 + DrawUtils.stringWidth(i2b.getName(), 13,
				GuiTextLabel.DEFAULTFONT, GuiTextLabel.DEFAULTSTILE);

		GuiTextLabel label = new GuiTextLabel(x + 54, y + getYMargin(inNum)
				+ heihgt / 2, i2b.getName(), 14);
		drawables.add(label);

		sections = new ArrayList<List<Point>>();
		points = new ArrayList<Point>();
		points.add(new Point(x + 39, y + getYMargin(inNum)));
		points.add(new Point(x + 55 + width, y + getYMargin(inNum)));
		points.add(new Point(x + 55 + width, y + getYMargin(inNum) + heihgt));
		points.add(new Point(x + 39, y + getYMargin(inNum) + heihgt));
		points.add(new Point(x + 39, y + getYMargin(inNum)));
		sections.add(points);
		drawables.add(new GuiLine(sections));

		int startI0 = getYMargin(inNum) + heihgt / 2 + inNum * DISTANCE / 2;

		for (int i = 0; i < inNum; i++) {
			sections = new ArrayList<List<Point>>();
			points = new ArrayList<Point>();
			points.add(new Point(x - 50 + 82, y + startI0));
			points.add(new Point(x - 50 + 89, y + startI0));
			sections.add(points);
			line = new GuiPinLine(sections, i2b.getInPin(i));
			drawables.add(line);
			inPortPosition = new Point(x - 50 + 89, y + startI0);

			label = new GuiTextLabel(x - 50 + 94, y + startI0, "" + i, 10);
			drawables.add(label);

			label = new GuiTextLabel(x + 28, y + startI0,
					i2b.getInPins()[i].getName(), 13, DrawUtils.ALIGNLEFT);
			drawables.add(label);

			label = new GuiPinLabel(x, y + startI0 + 15, i2b.getInPins()[i]);
			drawables.add(label);
			startI0 -= DISTANCE;
		}

		startI0 = getYEnd(outNum) - DISTANCE;

		for (int i = 0; i < outNum; i++) {
			sections = new ArrayList<List<Point>>();
			points = new ArrayList<Point>();
			points.add(new Point(x + 66 + width, y + startI0));
			points.add(new Point(x + 56 + width, y + startI0));
			sections.add(points);
			line = new GuiPinLine(sections, i2b.getOutPin(i));
			drawables.add(line);
			outPortPositions[i] = new Point(x + 66 + width, y + startI0);

			label = new GuiTextLabel(x + 40 + width, y + startI0 + 2, "" + i,
					10);
			drawables.add(label);
			if (printOutPortName) {
				label = new GuiTextLabel(x + 68 + width, y + startI0 + 2,
						i2b.getOutPins()[i].getName(), 13);
				drawables.add(label);
			}
			startI0 -= DISTANCE;
		}

		gui.addAll(drawables);
		return gui;
	}

	@Override
	public int getWidth() {
		return 150 + DrawUtils.stringWidth(component.getName(), 13,
				GuiTextLabel.DEFAULTFONT, GuiTextLabel.DEFAULTSTILE);
	}

	@Override
	public int getHeight() {
		int max = component.getOutPins().length;
		if (max <= 8) {
			return 180;
		} else {
			return 180 * max / 8;
		}
	}

	protected int getYMargin(int n) {
		if (n == 2) {
			return 46;
		} else {
			return 18;
		}
	}

	protected int getYEnd(int n) {
		int result = getYMargin(n);
		result += (n + 1) * DISTANCE;
		return result;
	}

	@Override
	public Point getOutPortPosition(int index) {
		if (index >= 0 && index < outPortPositions.length) {
			return outPortPositions[index];
		}
		return null;
	}

	@Override
	public Point getInPortPosition(int index) {
		if (index == 0) {
			return inPortPosition;
		}
		return null;
	}

}
