package logic.components;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import logic.Execution;
import logic.LogicalComponent;
import logic.Pin;
import util.Log;
import util.Mask;

public class MEM extends LogicalComponent {
	private Map<Integer, String> memory;
	private Pin read;
	private Pin write;

	private int adressSize;
	private int dataSize;

	public MEM(int adressSize, int dataSize) {
		super(2, 1);
		out[0].setIsInt();

		this.adressSize = adressSize;
		this.dataSize = dataSize;

		memory = new HashMap<Integer, String>();

		Execution.addSequentialComponent(this);

	}

	public void func() {
		if (write.getBoolVal()) {
			memory.put(in[0].getIntVal(),
					Integer.toString(in[1].getIntVal(), 16));
		}

		if (read.getBoolVal()) {
			String data = memory.get(in[0].getIntVal());
			if (data == null) {
				data = "0";
			}
			out[0].setIntVal(Integer.parseInt(data, 16));
		}
		if (!read.getBoolVal() && !write.getBoolVal()) {
			out[0].setIntVal(0);
		}
	}

	public void setRead(Pin read) {
		this.read = read;
		read.addChild(this);
	}

	public void setWrite(Pin write) {
		this.write = write;
		write.addChild(this);
	}

	public void write(int adress, int data) {
		memory.put(adress, Integer.toString(data, 16));
	}

	public int read(int adress) {
		String data = memory.get(adress);
		if (data == null) {
			data = "0";
		}
		return Integer.parseInt(data, 16);
	}

	public void initFromFile(String fileName) {
		BufferedReader reader = null;
		try {
			int addressMask = Mask.createMask(adressSize);
			int dataMask = Mask.createMask(dataSize);
			reader = new BufferedReader(new FileReader(fileName));
			String line;
			while ((line = reader.readLine()) != null) {
				try {

					line = removeComments(line);
					String[] linData = line.split(",");
					if (linData.length == 2) {
						String adr = linData[0].trim();
						int intAdr = Integer.parseInt(adr, 16);
						intAdr = intAdr & addressMask;

						String data = linData[1].trim();
						int intData = Integer.parseInt(data, 16);
						intData = intData & dataMask;

						memory.put(intAdr, Integer.toString(intData, 16));
					}
				} catch (Exception e) {
					// e.printStackTrace();
				}
			}
		} catch (Exception e) {
			Log.log(e, "Init from file " + fileName);
		} finally {
			try {
				reader.close();
			} catch (IOException e) {
				Log.log(e, "Closing file " + fileName);
			}
		}
	}

	public static String removeComments(String line) {
		if (line != null) {
			int index = line.indexOf("//");
			if (index > 0) {
				line = line.substring(0, index);
				line = line.trim();
			}
			line = line.toLowerCase().replace("\"", " ").trim();
		}
		return line;
	}

	public void init(boolean fullInit) {
		if (fullInit) {
			memory.clear();
		}
	}

	@Override
	public void initArgs(String[] args) {
		this.args = args;
		this.name = args[1];
	}

}
