package main;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import logic.LogicalComponent;
import logic.components.FF;
import util.ComponentNameComparator;
import util.NameConnector;

public class FlipflopDialogConfigurable extends JDialog {

	private static final long serialVersionUID = 1L;

	CPUFlipflopPanel panel;

	public FlipflopDialogConfigurable() {
		this.setTitle("Flip-flops");

		this.panel = new CPUFlipflopPanel(this, getFFs());
		this.add(panel);

		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent arg0) {
				setVisible(false);
				Main.currentScheme.repaint();
			}
		});

		this.setModal(true);

		this.setResizable(false);

		// this.setPreferredSize(new Dimension(600, 600));
		// this.setSize(600, 600);
		this.setLocation(100, 100);
		this.setVisible(false);

	}

	public void init() {
		panel.init();
	}

	class CPUFlipflopPanel extends JPanel implements ActionListener {
		private static final long serialVersionUID = 1L;
		private List<FF> ffs;
		private Map<String, FF> ffsNames;

		private List<JTextField> ffsFields;

		private JButton izmeni;
		private JButton izadji;

		private JDialog greska;
		private JDialog potvrda;

		private JDialog sadrzalacFFs;

		public JDialog getSadrzalacFFs() {
			return sadrzalacFFs;
		}

		public void setSadrzalacFFs(JDialog sadrzalacFFs) {
			this.sadrzalacFFs = sadrzalacFFs;
		}

		public CPUFlipflopPanel(FlipflopDialogConfigurable parent, List<FF> ffs) {
			super();
			sadrzalacFFs = parent;

			setLayout(new BorderLayout());
			setBackground(Color.white);

			this.ffs = ffs;
			this.ffsNames = new HashMap<String, FF>();
			for (FF ff : ffs) {
				ffsNames.put(ff.getName(), ff);
			}
			ffsFields = new LinkedList<JTextField>();

		}

		public void init() {
			Color bgcolor = Color.white;

			int numFFs = ffs.size();
			int numC = Math.max((int) (Math.sqrt(numFFs / 2)), 1);

			int numV = (int) (numFFs / numC);
			if (numC * numV < numFFs) {
				numV = numV + 1;
			}

			GridLayout layout = new GridLayout(1, 2 * numC);
			layout.setHgap(10);
			JPanel ffAll = new JPanel(layout);
			ffAll.setBackground(bgcolor);

			int index = 0;
			for (int i = 0; i < numC; i++) {
				GridLayout valuesLayout = new GridLayout(numV, 1);
				valuesLayout.setVgap(10);
				JPanel values = new JPanel(valuesLayout);
				values.setBackground(bgcolor);

				GridLayout labledLayout = new GridLayout(numV, 1);
				labledLayout.setVgap(10);
				JPanel lables = new JPanel(labledLayout);
				lables.setBackground(bgcolor);
				for (int j = 0; j < numV && index < numFFs; j++, index++) {

					FF ff = ffs.get(index);

					// JPanel tmpPanel = new JPanel();
					// tmpPanel.setBackground(bgcolor);

					JLabel ime = new JLabel(ff.getName());
					ime.setVerticalAlignment(0);
					ime.setHorizontalAlignment(4);
					lables.add(ime);

					JTextField ffPolje = new JTextField(5);// 5 je broj kolona
					ffPolje.setColumns(3);
					ffPolje.setText(Integer.toHexString(ff.getState()));
					ffPolje.setName(ff.getName());
					ffsFields.add(ffPolje);
					values.add(ffPolje);

					// prvi.add(tmpPanel);

				}
				ffAll.add(lables);
				// ffAll.add(Box.createVerticalStrut(0));
				ffAll.add(values);
			}

			add(ffAll, "Center");

			JPanel komande = new JPanel();
			komande.setBackground(bgcolor);

			izmeni = new JButton("Change");
			izmeni.addActionListener(this);

			izadji = new JButton("Cancel ");
			izadji.addActionListener(this);

			komande.add(this.izmeni);
			komande.add(this.izadji);
			add(komande, "South");
		}

		public void actionPerformed(ActionEvent arg) {

			if (arg.getActionCommand().equals("Change")) {
				boolean error = false;

				int cnt = ffs.size();
				boolean[] nizvrednosti = new boolean[cnt];
				for (int i = 0; i < cnt; i++) {
					nizvrednosti[i] = proveri(ffsFields.get(i));
					error = error || nizvrednosti[i];
				}

				if (!error) {
					for (int i = 0; i < cnt; i++) {
						JTextField ffField = ffsFields.get(i);
						FF ff = ffsNames.get(ffField.getName());

						ff.setState(dohvatiVrednost(ffsFields.get(i)));
					}
					potvrda = new JDialog();
					potvrda.setSize(350, 100);
					potvrda.setLocation(200, 200);
					potvrda.setModal(true);
					potvrda.setTitle("Potvrda");
					JPanel osnovni = new JPanel(new GridLayout(2, 1));
					JLabel lab = new JLabel(
							"Promena sadrzaja flip-flopova je uspesno odradjena!");
					lab.setHorizontalAlignment(0);
					osnovni.add(lab);
					JButton ok = new JButton("U redu");
					ok.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent arg0) {
							potvrda.setVisible(false);

						}
					});
					JPanel tmp = new JPanel();
					tmp.add(ok);
					tmp.setBackground(Color.white);
					osnovni.add(tmp);
					osnovni.setBackground(Color.white);
					potvrda.add(osnovni);
					potvrda.setVisible(true);

					Main.currentScheme.validate();
					Main.currentScheme.repaint();

				} else {
					error = false;
					greska = new JDialog();
					greska.setSize(550, 150);
					greska.setLocation(200, 200);
					greska.setModal(true);
					greska.setTitle("Greska!");
					JPanel osnovni = new JPanel(new GridLayout(3, 1));
					JLabel lab = new JLabel(
							"Neka od unetih vrednosti nije dobra (ili prelazi velicinu flip-flopa ili je unet netacan hex. broj).");
					lab.setHorizontalAlignment(0);
					osnovni.add(lab);
					JLabel lab1 = new JLabel(
							"Zato se brisu sve tekuce izmene flip-flopova!");
					lab1.setHorizontalAlignment(0);
					osnovni.add(lab1);
					JButton ok = new JButton("U redu");
					ok.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent arg0) {
							greska.setVisible(false);
							sadrzalacFFs.setVisible(true);
						}

					});
					JPanel tmp = new JPanel();
					tmp.add(ok);
					tmp.setBackground(Color.white);
					osnovni.add(tmp);
					osnovni.setBackground(Color.white);
					greska.add(osnovni);
					greska.setVisible(true);
				}
			} else {
				sadrzalacFFs.setVisible(false);
			}
			repaint();
		}

		private boolean proveri(JTextField field) {
			boolean error = false;
			String text = field.getText();
			int val;
			try {
				val = Integer.parseInt(text, 16);
			} catch (Exception e) { // ZBOG NUMBER FORMAT EXCEPTION!
				error = true;
				return error;
			}
			if (val != 0 && val != 1) {
				error = true;
			}
			return error;
		}

		private int dohvatiVrednost(JTextField field) {
			String text = field.getText();
			int val;
			try {
				val = Integer.parseInt(text, 16);
			} catch (Exception e) { // ZBOG NUMBER FORMAT EXCEPTION!
				return 0;
			}

			return val;
		}

		public void paint(Graphics g) {
			super.paint(g);

			for (JTextField field : ffsFields) {
				field.setText(FFFormater.format(ffsNames.get(field.getName())));
			}
		}
	}

	public static class FFFormater {

		public static int getSize(FF ff) {
			return 1;
		}

		public static String format(FF ff) {
			int validBits = 1;
			String result = String
					.format("%0" + validBits + "x", ff.getState());
			return result;
		}

		public static int reformat(String val, FF ff) {

			int result = 0;
			try {
				result = Integer.parseInt(val, 16);
			} catch (Exception e) {
			}
			return result;
		}
	}

	private List<FF> getFFs() {
		Set<FF> resultSet = new HashSet<FF>();
		Map<String, LogicalComponent> comps = NameConnector.listComponents();
		for (LogicalComponent comp : comps.values()) {
			if (comp instanceof FF) {
				resultSet.add((FF) comp);
			}
		}
		List<FF> result = new LinkedList<FF>(resultSet);
		Collections.sort(result, new ComponentNameComparator());
		return result;
	}

}
