package shemes.components;

import java.awt.Dimension;
import java.util.*;

import logic.LogicalComponent;
import logic.Pin;
import logic.components.*;
import shemes.AbstractSchema;
import util.*;
import gui.*;
import gui.components.*;

public class Fetch3 extends AbstractSchema {

	protected Map<String, LogicalComponent> components;

	public Fetch3() {
		componentName = "Fetch3";
		displayName = "Fetch 3";
		NameConnector.addSchema(componentName, this);
	}

	public void initComponent() {

		components = new HashMap<String, LogicalComponent>();
		for (int i = 0; i < Parameters.fetch3Conections.length; i++) {
			String[] data = Parameters.fetch3Conections[i];
			LogicalComponent component = loadComponent(data);
			component.initArgs(data);
			components.put(data[1], component);
		}

		putComponents();
		putPins();
	}

	private LogicalComponent loadComponent(String[] data) {
		LogicalComponent result = null;
		try {
			String type = data[0];
			String name = data[1];
			int size = Integer.parseInt(data[2]);
			if (type.equalsIgnoreCase("OR")) {
				result = new OR(size, name);
			} else if (type.equalsIgnoreCase("AND")) {
				result = new AND(size, name);
			} else if (type.equalsIgnoreCase("NAND")) {
				result = new NAND(size, name);
			} else if (type.equalsIgnoreCase("NOR")) {
				result = new NOR(size, name);
			} else if (type.equalsIgnoreCase("XOR")) {
				result = new XOR(size, name);
			} else if (type.equalsIgnoreCase("NOT")) {
				result = new NOT(name);
			} else if (type.equalsIgnoreCase("SPLIT")) {
				result = new IntToBools(size, name);
			}
		} catch (Exception e) {
		}
		return result;
	}

	public void initConections() {

		for (String[] data : Parameters.fetch3Conections) {
			LogicalComponent component = getLogicalComponent(data[1]);
			for (int i = 0; i < component.getInPins().length; i++) {
				Pin x = NameConnector.getPin(data[3 + i]);
				component.setInPin(i, x);
			}			
		}
	}

	private final int startX = 50;
	private final int startY = 0;

	public void initGui() {
		gui = new GuiSchema("src/images/Fetch3.png");

		List<LogicalComponent> comp = extractComponents();

		int numColumns = (int) Math.ceil(Math.sqrt(comp.size()));
		int numRows = (int) Math.ceil(comp.size() * 1.0 / numColumns);

		int y = startY;
		int cnt = 0;
		int maxX = 0;

		for (int i = 0; i < numRows && cnt < comp.size(); i++) {
			int max = 0;
			int x = startX;
			for (int j = 0; j < numColumns && cnt < comp.size(); j++) {
				LogicalComponent component = comp.get(cnt);
				DrawableComponent compGui = loadGuiComponent(component, x, y);
				gui.addDrawables(compGui.getDrawables());
				x += compGui.getWidth();
				maxX = x;
				if (max < compGui.getHeight()) {
					max = compGui.getHeight();
				}

				cnt++;
			}
			y += max;
		}
		Dimension dim = new Dimension(Math.max(maxX + 50, this.gui.getWidth()), Math.max(y + 50, this.gui.getHeight()));
		this.gui.adjustSize(dim);

	}

	private List<LogicalComponent> extractComponents() {
		List<LogicalComponent> result = new LinkedList<LogicalComponent>();
		List<LogicalComponent> comps = new LinkedList<LogicalComponent>(components.values());
		List<LogicalComponent> temp = new LinkedList<LogicalComponent>(components.values());

		for (LogicalComponent component : comps) {
			if (component instanceof AND) {
				if (component.getInPins().length == 2) {
					LogicalComponent left = getComponent(component.getInPin(0));
					LogicalComponent right = getComponent(component.getInPin(1));
					if ((left != null) && (right != null) && (left instanceof OR) && (right instanceof OR)) {
						TwoOne t = new TwoOne((OR) left, (OR) right, (AND) component);
						result.add(t);
						temp.remove(left);
						temp.remove(right);
					} else {
						result.add(component);
					}
				} else {
					result.add(component);
				}
				temp.remove(component);
			}
		}
		result.addAll(temp);
		return result;
	}

	private LogicalComponent getComponent(Pin out) {
		for (LogicalComponent component : components.values()) {
			for (Pin pin : component.getOutPins()) {
				if (out == pin) {
					return component;
				}
			}
		}
		return null;
	}

	private DrawableComponent loadGuiComponent(LogicalComponent component, int x, int y) {
		DrawableComponent compGui = null;
		if (component instanceof OR) {
			compGui = new OrGui(component, x, y, null, true, true);
		} else if (component instanceof AND) {
			compGui = new AndGui(component, x, y, null, true, true);
		} else if (component instanceof NAND) {
			compGui = new NandGui(component, x, y, null, true, true);
		} else if (component instanceof NOR) {
			compGui = new NorGui(component, x, y, null, true, true);
		} else if (component instanceof XOR) {
			compGui = new XorGui(component, x, y, null, true, true);
		} else if (component instanceof NOT) {
			compGui = new NotGui(component, x, y, null, true, true);
		} else if (component instanceof TwoOne) {
			compGui = new TwoOneGui(component, x, y, null, true, true);
		} else if (component instanceof IntToBools) {
			compGui = new IntToBoolsGui(component, x, y, null, true, true);
		}
		return compGui;
	}

	public void putPins() {

		for (LogicalComponent logicalComponent : components.values()) {
			NameConnector.addPins(componentName, logicalComponent.getOutPins());
		}

	}

	protected void putComponents() {

		for (String[] data : Parameters.fetch3Conections) {
			LogicalComponent component = getLogicalComponent(data[1]);
			component.initArgs(data);
		}

	}

	private LogicalComponent getLogicalComponent(String name) {
		return components.get(name);
	}

}
